function [x,y, hdid, hnext] = RKQC(ystart,dydx,NV,xsav,htry,EPS,yscal,...
                                 GG,Rho0,dgamdz,Alpha,Lambda)
%**************************************************}
% provide for the call the derivatives dydx at location x, given
% both x and the function values y. They must also define
% added ystart in the call on 8 Nov. 2019
   pgrow=-0.20;
   pshrnk=-0.25;
   fcor=0.06666666666666666666;   %(* 1.0/15.0 *)
   one=1.0;
   safety=0.9;
   errcon=6.0e-4;
   ysav = zeros(NV)'; dysav = ysav;
   for i = 1 : NV % do % begin
      % ysav(i) = y(i);
      ysav(i) = ystart(i);
      dysav(i) = dydx(i);
   end
   h = htry;
   errmax = 1.1; % added to start the while loop 
   x = xsav + 0.1; % added to start the while loop
   % while (errmax > one)
   while ((errmax > one) & (x-xsav)> EPS)
      hh = 0.5*h;
      % evaluate first Runge-Kutta for a step h/2:}
      ytemp = rk4(ysav,dysav,NV,xsav,hh,GG,Rho0,dgamdz,Alpha,Lambda);
      x = xsav+hh;
      dydx = DERIVS(x,ytemp,GG,Rho0,dgamdz,Alpha,Lambda);
      % second step starts at x=xsav+0.5*h. }
      % Evaluate second Runge-Kutta for a step h/2:}
      y = rk4(ytemp,dydx,NV,x,hh,GG,Rho0,dgamdz,Alpha,Lambda); % you have obtained y }
      x = xsav+h;
      if (abs(x-xsav) <= EPS) % then % begin
         input('stop in routine RKQC\r\n stepsize too small. Press Return to continue','s');
         % return;
      end
      % Evaluate Runge-Kutta for a full step h:}
      ytemp = rk4(ysav,dysav,NV,xsav,h,GG,Rho0,dgamdz,Alpha,Lambda);% ytemp by large step}
      errmax = 0.0;
      for i = 1 : NV % do % begin
         ytemp(i) = y(i)-ytemp(i);% difference in methods}
         temp = abs(ytemp(i)/yscal(i)); % relative error }
         if (errmax < temp) % then 
             errmax = temp;
         end    
      end
      errmax = errmax/EPS;
      % if (errmax > one) % then % begin
      h = safety*h*exp(pshrnk*log(errmax));
   end % while
   hdid = h;
   if (errmax > errcon) % then % begin
      hnext = safety*h*exp(pgrow*log(errmax));
   else % begin
      hnext = 4.0*h;
   end
   for i = 1 : NV
      y(i) = y(i)+ytemp(i)*fcor;
   end
end % RKQC }
