% Script splinmat made from PROGRAM SPLINRUN in Pascal
% Described in Malacic V.: Ecological Modeling, 138, 173-191, 2001 and 
% Malacic V., 1998. Geofizikalno-ekoloki pristop k disperziji odplak piranskega izpusta. 
% Fazno porocilo 2, Nacionalni intitut za biologijo, Morska bioloka postaja Piran, 43 str.
      NN = 500; % maximum dimension of arrays, also number of steps in Runge-Kutta}
      kmax = 400;%200}; % max. number of results, kount < kmax}
      NV = 4; % Num. of  variables }

% An Introduction to Water Quality Modelling, A.James (Ed.), Wiley, 1984, 234 pp., 
% page 155: Lambda = 1.16;  Fischer et al., 1979, Mixing in Inland Waters and Coastal Waters,
% Academic Press, page 370: Lambda = 1.2
      Lambda = 1.14; % Lambda = ratio between the radius at which the density 
      % difference (ambient density minus core density of the buoyant jet) falls 
      % to 1/e of the core density difference, and the radius at which the 
      % velocity falls to 1/e of the core velocity in a buoyant slice. In both, the density 
      % difference profile and the velocity profile across the buoyant jet the Gaussian profile is supposed. 
      % for Lamba =1.14 the best numerical results were obtained when compared with semiempirical solutions.
      ystart = zeros(NV,1);% starting values of variables; glnarray;
      % yp = size NN x NV = results, four output variables, see below 'initial data'
      time = zeros(NN,1);
% begin % MAIN program  }
%**************************************************}
   varyAlpha = input('Do you want to vary Alpha (the enrtainment coefficient) as a function of Ri number? Y/N [N]:','s');
   % Fischer et al., 1979, Mixing in Inland Waters and Coastal Waters, Academic Press
   if isempty(varyAlpha)
      varyAlpha = 'N';
   end
   varyAlpha = upper(varyAlpha);
   
   DepthDifstr = '10.1'; % DepthDif = 10.1; %36.0;}%20.7;} % Depth of diffusser
   Diameterstr = '0.1';%'0.6';%0.1;}%3.0;;} % [m] diameter of the orifice } 0.6 m = no difussor
   u0str = '12.7';%'3.54';%12.7;}%20.0} %1.27;} %0.5;} % [m/s] 3.54 m/s = no difussor, 1000 l/s
   theta0str =  '0';%PI*(90.0)/180.} PI*(0.0)/180.;   
   prompt = {'Enter diffusor depth (m):','Enter orifice diameter (m):',...
             'Enter initial speed from the orifice (m/s):',...
             'Enter initial jet inclination above the horizontal plane (degrees):'};
   dlgtitle = 'Input params';  
   dims = [1 30; 1 35; 1 45; 1 70];% [1 70]
   definput = {DepthDifstr,Diameterstr,u0str,theta0str};% dif. depth(m), orifice diam.(m), u0(m/s), theta(deg)
   replies = inputdlg(prompt, dlgtitle, dims,definput);
   initstrngs = replies;
   % Boris initstrngs = string(replies);
   DepthDif = str2double(initstrngs(1));
   Diameter = str2double(initstrngs(2));
   u0 = str2double(initstrngs(3));
   theta0 = str2double(initstrngs(4));
   theta0 = theta0*pi/180;
   SS0 = 6.2*Diameter;
   s1 = 0.001; % guessed first stepsize }
   smin = 1.0E-6; % minimum step size = 1 micron, is 'small enough'}
   SSmax = 100.0;% not known final distance along the jet }
   dxsav = 0.1;%recorded points are separated by ds > 0.1 }
   b0 = Diameter/2^0.5; % Water Quality Model, p.155 }
   bmax = 100,0; % maximum radius of the buoyant jet - chosen arbitrary to close the 
   % program if radius of the jet becomes too large before the 'final'
   % depth (or sea surface) is achieved
   [NumDat,HeightArr,GamArr] = ReadData(DepthDif);

   %***** GET DENSITY AT INITIAL HEIGHT gam0 BY INTERPOLATION FROM THE DENSITY PROFILE *****}
   Height0 = eps; % starting height above the diffusser }
   tempklo0 = find((HeightArr(1:NumDat) <= Height0),1,'last')
   % get coefficients of polynoms in each interval withing points HeightArr
  
   YesMakima = input('Do you want to apply the Makima spline (Matlab R2019b) Y/N. Default [N]:','s');
   if isempty(YesMakima)
      YesMakima = 'N';
   end
   YesMakima = upper(YesMakima);

   % get the interpolated value of density gam0 and its vertical derivative 
   % dgamdz at the diffuser height Height0 
   if YesMakima == 'Y'
      ppmakima = makima(HeightArr,GamArr);
      %https://blogs.mathworks.com/cleve/2019/04/29/makima-piecewise-cubic-interpolation/
      [gam0, dgamdz0]=ppvalder(ppmakima,Height0);
   else
      ppsplin = spline(HeightArr,[0 GamArr' 0]);
      % [0 GamArr 0] added zero first derivatives in the first and the last data point     
      [gam0, dgamdz0]=ppvalder(ppsplin,Height0);
   end   
   
   % 
   fprintf(1,'HeightArr(%d)= %6.2f Height0 = %6.2f gam0= %7.3f dgamdz0= %7.3f\r\n',...
           tempklo0,HeightArr(tempklo0),Height0, gam0, dgamdz0);     
   %************   INITIAL DATA  *******************}
   % SS0 starting distance [m] along the jet }
   % ystart[1] = velocity along the jet axis }
   % ystart[2] = half-width of a buoyant jet }
   % ystart[3] = angle of jet axis with respect to XX }
   % ystart[4] = delta_rho = rho_ambient - rho_jet }
   ystart(1) = u0;
   ystart(2) = b0;
   ystart(3) = theta0;
   ystart(4) = gam0*(1.0 + (Lambda)^2)/(2.0*(Lambda)^2);
   
   % suppose a sewage with the density of 1000 kg/m^3,  }
   % delta_Rho0 = gam0, also 'Water Quality Model.',1984, p.154 }
   Rho0 = 1000.0 + gam0;%Rho0 = 995.8115 + gam0;} %T=30 st.C, S=0, p=36 dbar}
   

   %*********   NAMES OF OUTPUT FILES ************}
   prompt = {'Enter the output file name for buoyant jet drawing:'};
   dlgtitle = 'output data for drawing';
   dims = [1 35];
   defoutdrw = {'outputdraw.txt'};
   answer = inputdlg(prompt,dlgtitle,dims,defoutdrw);
   fiddrw = fopen(char(answer),'w+');
   % Boris fiddrw = fopen(string(answer),'w+');
   
   prompt = {'Enter the output file name for key output data:'};
   dlgtitle = 'output key data';
   dims = [1 35];
   defoutdrw = {'outputkey.txt'};
   answer = inputdlg(prompt,dlgtitle,dims,defoutdrw);   
   fidkey = fopen(char(answer),'w+');
   % Boris fidkey = fopen(string(answer),'w+');
   
   prompt = {'Enter the output file name for comparing num and analytical results:'};
   dlgtitle = 'output data for comparison';
   dims = [1 35];
   defoutdrw = {'outputcom.txt'};
   answer = inputdlg(prompt,dlgtitle,dims,defoutdrw);
   fidcom = fopen(char(answer),'w+');   
   % Boris fidcom = fopen(string(answer),'w+');
  
%(***   CALL THE BEST FAECALI ENGINE:   *****)
%**************************************************}
   [nok,nbad,kount,xp,yp,xxp,zzp,RiArr,AlphArr] = FAECALI(ystart,...
                  NV,kmax,...
                  HeightArr,GamArr,...
                  varyAlpha,DepthDif,...
                  SS0,SSmax,s1,smin,bmax,dxsav,...
                  Lambda,Diameter,Height0,theta0,YesMakima);
   
   fprintf(1,'success. steps (nok)= %d  bad steps (nbad)= %d stored interm. values (kount) = %d\r\n',...
           nok,nbad,kount);
   xp = xp(1:kount); xxp = xxp(1:kount); zzp = zzp(1:kount); % shorten arrays to results
   RiArr = RiArr(1:kount); AlphArr = AlphArr(1:kount);
   yp = yp(1:kount,:);
   for i =2 : kount % kount 
      time(i) = time(i-1) + ...
      (xp(i)-xp(i-1))*2.0/(yp((i),1)+yp(i-1,1));
   end
   % time = time(1:kount); 
   time = time(1:kount); 
   
%(***   Call the STRATIMIDSURF function to write down results   *****)
%**************************************************} 
   [Sm,SSstra,ZZstra,minkount] = STRATIMIDSURF(xp,yp,time,xxp,zzp,NV,kount,...
                        Rho0,Height0,DepthDif,RiArr,AlphArr,Lambda,Diameter,u0,...
                        HeightArr,GamArr,fiddrw,fidkey,fidcom,YesMakima);
   % Sm = scalar, the last value of numerical dilution. SSstra = calculated dilution
   % from analytic expression for a linear vertical density profile. ZZstra = max height of sewage, 
   % calculated analytiucally from a linear vertical density profile.
   % if zzp(kount) < zmax = DepthDif 
   %      then zzp(del_rho=0) IS WITHIN the WATER COLUMN
   % else at the surface 
   fprintf(1,'After STRATIMIDSURF kount=%d\r\n', kount);
   fclose(fiddrw);
   fclose(fidkey);
   fclose(fidcom);
 %*************************** put data on screen and plot the data  
   disp('  time(s)    xp(m)    yp1(m/s)    yp2(m)   yp3(rad)  yp4(kg/m3)  xxp(m)   zzp(m)');
   [time(1:10) xp(1:10) yp(1:10,:) xxp(1:10) zzp(1:10)]
   disp('  time(s)    xp(m)    yp1(m/s)    yp2(m)   yp3(rad)  yp4(kg/m3)  xxp(m)   zzp(m)');
   [time(kount-1:kount) xp(kount-1:kount) yp(kount-1:kount,:) xxp(kount-1:kount) zzp(kount-1:kount)]
   figure(1)
   plot(time(1:kount),yp(1:kount,1),'k',time(1:kount),yp(1:kount,2),'b',...
        time(1:kount),yp(1:kount,3).*(180/pi),'g',time(1:kount),yp(1:kount,4),'r','LineWidth',2 )
   xlabel('{\it{time}}(s)');
   legend('\it{u}(m/s)','\it{b}(m)','\theta ()','{\Delta\rho}(kg/m^3)');
   grid on;
   figure(2)
   xxpbr = xxp+yp(:,2).*sin(yp(:,3)); zzpbr = zzp-yp(:,2).*cos(yp(:,3)); % br = bottom right
   indbr = find(zzpbr <= DepthDif,1,'last'); xxpbr = xxpbr(1:indbr); zzpbr = zzpbr(1:indbr); 
   xxptl = xxp-yp(:,2).*sin(yp(:,3)); zzptl =  zzp+yp(:,2).*cos(yp(:,3)); % tl = top left
   indtl= find(zzptl <= DepthDif,1,'last'); xxptl = xxptl(1:indtl); zzptl = zzptl(1:indtl); 
   indc = find(zzp <= DepthDif,1,'last'); % c = central
   xxpc = xxp(1:indc); zzpc = zzp(1:indc);
   plot(xxpc,zzpc,'r',xxpbr,zzpbr,'b:',xxptl,zzptl,'b:','LineWidth',2);
   xlabel('{\it{x}}(m)');ylabel('{\it{z}}(m)');
   grid on; axis equal;
   figure(3)
   plot(time,zzp,'LineWidth',2)
   xlabel('{\it{time}}(s)');ylabel('{\it{z}}(m)');
   grid on;
   
   
